from easy_tg_bot import get_secret_by_name, default_roles

# Convention
# ! easy_tg_bot run --upd-env clears the env variables that start with "TG_"

# Telegram IDs of (super)admins
# https://t.me/getmyid_bot
TG_MY_ID = str(get_secret_by_name("TG_MY_ID"))

# Default roles
# Possible keys: stuff in you .env or env variable with the name 
# Possible values: superadmin, admin, user, banned
ROLES = {
    "TG_MY_ID": "superadmin"
}

# Add default roles 
default_roles(ROLES)



import os
from dotenv import load_dotenv

from .utils.init_templates import initialize_file_from_draft


# Get sectret
def get_secret_by_name(name: str, default=None):
    result = os.getenv(name)
    if not result and default is None: 
        if not os.path.exists(os.path.join(os.getcwd(), ".env")):
            initialize_file_from_draft(".env", os.getcwd())
            raise Exception("No .env file found in the current working directory. Please fill .env file.")
        env_result = load_dotenv(dotenv_path=os.path.join(os.getcwd(), ".env"), override=True, verbose=True)
        if not env_result:
            raise ValueError("Error initializing .env")
        result = os.getenv(name)
        if not result:
            raise ValueError(f"Error initializing env variable: {name}")
    return result or default

# TG creds
TG_BOT_NAME = get_secret_by_name("TG_BOT_NAME")
TG_BOT_TOKEN = get_secret_by_name("TG_BOT_TOKEN")

# Optional
TG_TIME_ZONE = get_secret_by_name("TG_TIME_ZONE", "")

# File storage/mount
TG_FILE_FOLDER_PATH = get_secret_by_name("TG_FILE_FOLDER_PATH", "")

# Webhook
TG_WEBHOOK_URL = get_secret_by_name(
    "TG_WEBHOOK_URL", ""
)
TG_WEBHOOK_HOST = get_secret_by_name("TG_WEBHOOK_HOST", "0.0.0.0")
TG_WEBHOOK_PORT = get_secret_by_name("TG_WEBHOOK_PORT", 8000)

# Roles
DEFAULT_ROLES_DICT = {}

# Possible values: superadmin, admin, user, banned
def default_roles(roles):
    DEFAULT_ROLES_DICT.update({str(get_secret_by_name(k)): v for k, v in roles.items()})

def get_default_role(user_id):
    return DEFAULT_ROLES_DICT.get(str(user_id), "user")



import contextlib
import asyncio
import json
import os

from telegram.ext import Application, ApplicationBuilder, PicklePersistence
from telegram import Update

from fastapi import FastAPI, Request, Response
from http import HTTPStatus
import uvicorn
from pyngrok import ngrok

# Suppress uvicorn logger
import logging
uvicorn_logger = logging.getLogger("uvicorn")
uvicorn_logger.setLevel(logging.ERROR)

# Suppress pyngrok and fastapi loggers
pyngrok_logger = logging.getLogger("pyngrok")
pyngrok_logger.setLevel(logging.ERROR)

fastapi_logger = logging.getLogger("fastapi")
fastapi_logger.setLevel(logging.ERROR)

from .utils.init_templates import initialize_file_from_draft
from .mylogging import time_log_decorator, logger
from . import settings

# add handlers just with decorators throughout the code
from .decorators import add_handlers


# -------------------------------------------TELEGRAM BOT DB------------------------------
class MyApplication(Application):
    # Disagreeing with the user/chat_data read-only policy
    def __init__(
        self,
        *args,
        **kwargs,
    ):
        super().__init__(*args, **kwargs)
        self.user_data = self._user_data
        self.chat_data = self._chat_data

    async def stop(self):
        # TODO handle urgent shutdown
        # Ensure all user data and chat data are saved
        if self.persistence:
            for user_id, data in self.user_data.items():
                await self.persistence.update_user_data(user_id, data)
            for chat_id, data in self.chat_data.items():
                # Sanitize for unpickable objects: convention
                for k in list(data.keys()):
                    if k.endswith("_func"):
                        data.pop(k)
                await self.persistence.update_chat_data(chat_id, data)

        # Call the original stop method
        await super().stop()


# --------------------------------------------TELEGRAM APP---------------------------------
def prepare_app():
    root_dir = settings.TG_FILE_FOLDER_PATH or os.getcwd() 
    initialize_file_from_draft("text.xlsx", root_dir)
    persistence = PicklePersistence(
        filepath=f"{settings.TG_FILE_FOLDER_PATH}{settings.TG_BOT_NAME}Persistence", on_flush=True
    )
    application = (
        ApplicationBuilder()
        .application_class(MyApplication)
        .token(settings.TG_BOT_TOKEN)
        .persistence(persistence)
        .build()
    )

    return application

# App instance
application = prepare_app()

# Operations
@time_log_decorator
def telegram_bot_polling(debug = False):
    # Add handlers; needs to be called at the end for the decorators reason
    add_handlers(application, debug)
    # Ignore exception when Ctrl-C is pressed
    with contextlib.suppress(KeyboardInterrupt):  
        application.run_polling()


# AWS
@time_log_decorator
async def process_update(event, context):
    # Add handlers; needs to be called at the end for the decorators reason
    add_handlers(application)
    try:
        await application.initialize() # .start() ?
        await application.process_update(
            Update.de_json(json.loads(event["body"]), application.bot)
        )
        await application.stop()
        return {"statusCode": 200, "body": "Success"}

    except Exception as e:
        logger.error(f"Error in process_update: {e}")
        return {"statusCode": 500, "body": "Failure"}


def lambda_handler(event, context):
    return asyncio.get_event_loop().run_until_complete(process_update(event, context))


# Vultr and others
def ptb_for_webhook():
    persistence = PicklePersistence(
        filepath=f"{settings.TG_FILE_FOLDER_PATH}{settings.TG_BOT_NAME}Persistence",
        on_flush=True,
    )
    ptb = (
        ApplicationBuilder()
        .application_class(MyApplication)
        .updater(None)
        .token(settings.TG_BOT_TOKEN)
        .persistence(persistence)
        .read_timeout(7)
        .get_updates_read_timeout(42)
        .build()
    )
    add_handlers(ptb)
    return ptb

def app_for_webhook(url):
    root_dir = settings.TG_FILE_FOLDER_PATH or os.getcwd()
    initialize_file_from_draft("text.xlsx", root_dir)

    @contextlib.asynccontextmanager
    async def lifespan(app: FastAPI):
        await app.ptb.bot.setWebhook(url)
        async with app.ptb as ptb:
            await ptb.start()
            yield
            await ptb.stop()

    # Initialize FastAPI app (similar to Flask)
    app = FastAPI(lifespan=lifespan)
    # TODO Limiter
    app.ptb = ptb_for_webhook()

    @app.post("/")
    async def process_update(request: Request):
        req = await request.json()
        update = Update.de_json(req, app.ptb.bot)
        try:
            await app.ptb.process_update(update)
            return Response(status_code=HTTPStatus.OK)
        except Exception as e:
            logger.error(f"Error in process_update: {e}")
            return Response(status_code=HTTPStatus.INTERNAL_SERVER_ERROR)
    return app

def run_webhook_app():
    host = settings.TG_WEBHOOK_HOST
    port = settings.TG_WEBHOOK_PORT
    token = settings.TG_BOT_TOKEN
    url = settings.TG_WEBHOOK_URL
    if not url:
        logger.info("No url found, setting up ngrok...")
        url = ngrok.connect(port).public_url
    if not all([host, port, token]):
        raise ValueError("Please fill all the necessary settings in settings.py")
    if not url:
        raise ValueError("Please check your ngrok connection")
    app = app_for_webhook(url)
    uvicorn.run(app, host=host, port=port, access_log=False)



import click
import os

from .mylogging import logger
from .utils.init_templates import initialize_file_from_draft, create_vultr_deploy_yml_from_draft
from . import settings
from .utils.run_docker_polling import run_container, create_yaml_file
from .app import run_webhook_app


# Main
@click.group()
@click.version_option()
def cli():
    """The CLI tool."""


# Utils
def clear_env_variables():
    """
    Clears all environment variables set by the .env file.
    Relevant keys start with "TG_".
    """
    for key in os.environ:
        if key.startswith("TG_"):
            os.putenv(key, "")
            # print(f"CLEARED ENV: {key}")


# Builds neccessary files for the developer and runs polling
# @click.option("--upd-env", is_flag=True, help="Clear old env. var. configuration")
@click.option("--docker", is_flag=True, help="Deploy as a Docker container; env vars from .env")
@cli.command()
def run(docker):
    # make it read .env file every time at least for TG_
    clear_env_variables()

    root_dir = settings.TG_FILE_FOLDER_PATH or os.getcwd()
    initialize_file_from_draft("text.xlsx", root_dir)

    root_dir = os.getcwd()
    initialize_file_from_draft("settings.py", root_dir)
    initialize_file_from_draft("main.py", root_dir)
    initialize_file_from_draft(".gitignore", root_dir)
    if docker:
        initialize_file_from_draft("Dockerfile", root_dir)
        initialize_file_from_draft(".dockerignore", root_dir)
        create_yaml_file()
        run_container()
        return
    # poetry ads
    try:
        os.system("poetry run python main.py")
    except Exception as e:
        logger.error(f"Please install poetry, or launch main.py directly yourself. Sorry! Error:\n{e}")


# Set the webhook for a Telegram Bot
@click.option("--token", default=settings.TG_BOT_TOKEN)
@click.option("--url", default=settings.TG_WEBHOOK_URL)
@cli.command()
def set_webhook(token, url):
    os.system(f"curl -X POST https://api.telegram.org/bot{token}/setWebhook?url={url}")


@cli.command()
def webhook():
    try:
        run_webhook_app()
    except Exception as e:
        logger.error(f"Error in webhook:\n{e}")


@cli.command()
def vultr():
    """Deploys to Vultr using Actions."""
    root_dir = os.getcwd()
    initialize_file_from_draft("Dockerfile", root_dir)
    initialize_file_from_draft(".dockerignore", root_dir)
    create_vultr_deploy_yml_from_draft()


# Developer mode
if __name__ == "__main__":
    cli()



import yaml
import os
from .. import settings


IMG_NAME = str(settings.TG_BOT_NAME).lower()


# Utils
def create_yaml_file(version="3.12", image_name=IMG_NAME):
    # Define the structure of your Docker Compose YAML
    keys = read_env_keys()
    docker_compose_config = {
        "version": version,
        "name": image_name,
        "services": {
            "app": {
                "image": f"{image_name}",
                "environment": [
                    f"{key}=${{{key}}}" for key in keys
                ],
            }
        },
    }
    # Write to a file
    path = os.path.join(os.getcwd(), "docker-compose.yml")
    with open(path, "w") as file:
        yaml.dump(docker_compose_config, file, default_flow_style=False)
    print("Docker Compose YAML file has been created.")


def read_env_keys():
    """Load environment variables from a .env file."""
    env_keys = []
    path = os.path.join(os.getcwd(), ".env")
    with open(path, "r") as file:
        for line in file:
            if line.strip() and not line.startswith("#"):
                key, _ = line.strip().split("=", 1)
                env_keys.append(key)
    return env_keys


def run_container():
    dockerfile = os.path.join(os.getcwd(), "Dockerfile")
    tag=f"{IMG_NAME}:latest"
    context = "."
    build_command = f"docker build -f {dockerfile} -t {tag} {context}"
    print(f"Executing: {build_command}")
    build_status = os.system(build_command)
    if build_status != 0:
        print("Failed to build Docker image.")
        return
    run_command = f"docker-compose up"
    print(f"Executing: {run_command}")
    run_status = os.system(run_command)
    if run_status != 0:
        print("Failed to run Docker container.")
        return
    print("Container started successfully.")
